@@ -9,7 +9,7 @@
 package Image::ExifTool::XMP;
 
 use strict;
-use vars qw(%specialStruct $xlatNamespace);
+use vars qw(%specialStruct %stdXlatNS);
 
 use Image::ExifTool qw(:Utils);
 use Image::ExifTool::XMP;
@@ -83,7 +83,7 @@ sub InflateStruct($;$)
                 ($part = $$obj) =~ s/^\s*//s;
                 $part =~ s/[\x0d\x0a].*//s;
                 $part = substr($part,0,27) . '...' if length($part) > 30;
-                $warn = "Invalid structure field at '$part'";
+                $warn = "Invalid structure field at '${part}'";
             } else {
                 $warn = 'Missing closing brace for structure';
             }
@@ -223,7 +223,7 @@ Key:
                     $tagInfo = $ti;
                     $g1 = $grps[1];
                 }
-                $tagInfo or $warn =  "'$tag' is not a writable XMP tag", next Key;
+                $tagInfo or $warn =  "'${tag}' is not a writable XMP tag", next Key;
                 GetPropertyPath($tagInfo);  # make sure property path is generated for this tag
                 $tag = $$tagInfo{Name};
                 $tag = "$g1:$tag" if $grp;
@@ -264,7 +264,7 @@ Key:
                     last; # write this lang-alt field
                 }
             }
-            $warn = "'$tag' is not a field of $strName";
+            $warn = "'${tag}' is not a field of $strName";
             next Key;
         }
         if (ref $$struct{$key} eq 'HASH') {
@@ -281,12 +281,18 @@ Key:
             foreach $item (@{$$struct{$key}}) {
                 if (not ref $item) {
                     $item = '' unless defined $item; # use empty string for missing items
-                    $$fieldInfo{Struct} and $warn = "$tag items are not valid structures", next Key;
-                    $et->Sanitize(\$item);
-                    ($copy[$i],$err) = $et->ConvInv($item,$fieldInfo,$tag,$strName,$type,'');
-                    $err and $warn = $err, next Key;
-                    $err = CheckXMP($et, $fieldInfo, \$copy[$i]);
-                    $err and $warn = "$err in $strName $tag", next Key;
+                    if ($$fieldInfo{Struct}) {
+                        # (allow empty structures)
+                        $item =~ /^\s*$/ or $warn = "$tag items are not valid structures", next Key;
+                        $copy[$i] = { }; # create hash for empty structure
+                    } else {
+                        $et->Sanitize(\$item);
+                        ($copy[$i],$err) = $et->ConvInv($item,$fieldInfo,$tag,$strName,$type,'');
+                        $copy[$i] = '' unless defined $copy[$i];    # avoid undefined item
+                        $err and $warn = $err, next Key;
+                        $err = CheckXMP($et, $fieldInfo, \$copy[$i]);
+                        $err and $warn = "$err in $strName $tag", next Key;
+                    }
                 } elsif (ref $item eq 'HASH') {
                     $$fieldInfo{Struct} or $warn = "$tag is not a structure in $strName", next Key;
                     ($copy[$i], $err) = CheckStruct($et, $item, $$fieldInfo{Struct});
@@ -304,18 +310,19 @@ Key:
             $et->Sanitize(\$$struct{$key});
             ($val,$err) = $et->ConvInv($$struct{$key},$fieldInfo,$tag,$strName,$type,'');
             $err and $warn = $err, next Key;
+            next Key unless defined $val;   # check for undefined
             $err = CheckXMP($et, $fieldInfo, \$val);
             $err and $warn = "$err in $strName $tag", next Key;
             # turn this into a list if necessary
             $copy{$tag} = $$fieldInfo{List} ? [ $val ] : $val;
         }
     }
-    if (%copy) {
+    if (%copy or not $warn) {
         $rtnVal = \%copy;
         undef $err;
         $$et{CHECK_WARN} = $warn if $warn;
     } else {
-        $err = $warn || 'Structure has no fields';
+        $err = $warn;
     }
     return wantarray ? ($rtnVal, $err) : $rtnVal;
 }
@@ -474,8 +481,16 @@ sub AddNewStruct($$$$$$)
     my $ns = $$strTable{NAMESPACE} || '';
     my $changed = 0;
 
+    # add dummy field to allow empty structures (name starts with '~' so it will come
+    # after all valid structure fields, which is necessary when serializing the XMP later)
+    %$struct or $$struct{'~dummy~'} = '';
+
     foreach $tag (sort keys %$struct) {
-        my $fieldInfo = $$strTable{$tag} or next;
+        my $fieldInfo = $$strTable{$tag};
+        unless ($fieldInfo) {
+            next unless $tag eq '~dummy~'; # check for dummy field
+            $fieldInfo = { }; # create dummy field info for dummy structure
+        }
         my $val = $$struct{$tag};
         my $propPath = $$fieldInfo{PropertyPath};
         unless ($propPath) {
@@ -584,7 +599,7 @@ sub ConvertStruct($$$$;$)
             return $val;
         } else {
             my (@list, $val);
-            foreach $val (@$value) {    
+            foreach $val (@$value) {
                 my $v = ConvertStruct($et, $tagInfo, $val, $type, $parentID);
                 push @list, $v if defined $v;
             }
@@ -603,7 +618,7 @@ sub RestoreStruct($;$)
 {
     local $_;
     my ($et, $keepFlat) = @_;
-    my ($key, %structs, %var, %lists, $si, %listKeys);
+    my ($key, %structs, %var, %lists, $si, %listKeys, @siList);
     my $ex = $$et{TAG_EXTRA};
     my $valueHash = $$et{VALUE};
     my $tagExtra = $$et{TAG_EXTRA};
@@ -623,17 +638,19 @@ sub RestoreStruct($;$)
                 # this could happen for invalid XMP containing mixed lists
                 # (or for something like this -- what should we do here?:
                 # <meta:user-defined meta:name="License">test</meta:user-defined>)
-                $et->Warn("$$strInfo{Name} is not a structure!");
+                $et->Warn("$$strInfo{Name} is not a structure!") unless $$et{NO_STRUCT_WARN};
                 next;
             }
         } else {
             # create new entry in tag table for this structure
             my $g1 = $$table{GROUPS}{0} || 'XMP';
             my $name = $tag;
+            # tag keys will have a group 1 prefix when coming from import of XML from -X option
             if ($tag =~ /(.+):(.+)/) {
                 my $ns;
                 ($ns, $name) = ($1, $2);
-                $ns = $$xlatNamespace{$ns} if $$xlatNamespace{$ns};
+                $ns =~ s/^XMP-//; # remove leading "XMP-" if it exists because we add it later
+                $ns = $stdXlatNS{$ns} if $stdXlatNS{$ns};
                 $g1 .= "-$ns";
             }
             $strInfo = {
@@ -725,7 +742,7 @@ sub RestoreStruct($;$)
                 # XMP namespace on the tag name.  In this case, add
                 # the corresponding group1 name to the tag ID.
                 my ($ns, $name) = ($1, $2);
-                $ns = $$xlatNamespace{$ns} if $$xlatNamespace{$ns};
+                $ns = $stdXlatNS{$ns} if $stdXlatNS{$ns};
                 $tag = "XMP-$ns:" . ucfirst $name;
             } else {
                 $tag = ucfirst $tag;
@@ -734,18 +751,22 @@ sub RestoreStruct($;$)
         if ($err) {
             # this may happen if we have a structural error in the XMP
             # (like an improperly contained list for example)
-            my $ns = $$tagInfo{Namespace} || $$tagInfo{Table}{NAMESPACE} || '';
-            $et->Warn("Error $err placing $ns:$$tagInfo{TagID} in structure or list", 1);
+            unless ($$et{NO_STRUCT_WARN}) {
+                my $ns = $$tagInfo{Namespace} || $$tagInfo{Table}{NAMESPACE} || '';
+                $et->Warn("Error $err placing $ns:$$tagInfo{TagID} in structure or list", 1);
+            }
             delete $structs{$strInfo} unless $oldStruct;
         } elsif ($tagInfo eq $strInfo) {
-            # just a regular list tag
+            # just a regular list tag (or an empty structure)
             if ($oldStruct) {
                 # keep tag with lowest numbered key (well, not exactly, since
                 # "Tag (10)" is lt "Tag (2)", but at least "Tag" is lt
                 # everything else, and this is really what we care about)
                 my $k = $listKeys{$oldStruct};
-                $k lt $key and $et->DeleteTag($key), next;
-                $et->DeleteTag($k);   # remove tag with greater copy number
+                if ($k) {   # ($k will be undef for an empty structure)
+                    $k lt $key and $et->DeleteTag($key), next;
+                    $et->DeleteTag($k);   # remove tag with greater copy number
+                }
             }
             # replace existing value with new list
             $$valueHash{$key} = $structs{$strInfo};
@@ -782,9 +803,10 @@ sub RestoreStruct($;$)
     foreach $si (keys %lists) {
         defined $_ or $_ = '' foreach @{$lists{$si}};
     }
-    # save new structure tags
-    foreach $si (keys %structs) {
-        next unless $var{$si};  # already handled regular lists
+    # make a list of all new structures we generated
+    $var{$_} and push @siList, $_ foreach keys %structs;
+    # save new structures in the same order they were read from file
+    foreach $si (sort { $var{$a}[1] <=> $var{$b}[1] } @siList) {
         $key = $et->FoundTag($var{$si}[0], '');
         $$valueHash{$key} = $structs{$si};
         $$et{FILE_ORDER}{$key} = $var{$si}[1];
@@ -811,7 +833,7 @@ information.
 
 =head1 AUTHOR
 
-Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
